from random import randint
from param import TAILLE, TYPE_MOUTON, TYPE_LOUP
from analyse_monde import est_vide

"""
Ce fichier définie toutes les fonctions permettant de créer un monde
peuplé de moutons et de loups.
Le monde est défini par une grille carrée (la taille de la grille est la
constante TAILLE du fichier param.py), dont chaque case peut contenir
au plus un animal, qui peut être un loup ou un mouton.
"""
def creer_mouton(numero, abscisse, ordonnee):
    """
    Un mouton est représenté par 6 caractéristiques :
      - son type (le fait que c'est un mouton)
      - son numéro (pour différencier les moutons si nécessaire"
      - l'abscisse et l'ordonnée de sa position dans le monde
      - des informations éventuelles : cette case peut être
      utilisée pour stocker d'éventuelles informations propres
      au mouton
      - son niveau de stress, initialisé à 0
    """
    return [TYPE_MOUTON, numero, abscisse, ordonnee, None, 0]

def creer_loup(numero, abscisse, ordonnee):
    """
    Un loup est représenté par 5 caractéristiques :
      - son type (le fait que c'est un loup)
      - son numéro (pour différencier les loups si nécessaire"
      - l'abscisse et l'ordonnée de sa position dans le monde
      - des informations éventuelles : cette case peut être
      utilisée pour stocker d'éventuelles informations propres
      au loup
    """
    return ['loup', numero, abscisse, ordonnee, None]

def creer_moutons(monde, n):
    """
    Fonction permettant de peupler le monde (passé en premier paramètre)
    avec un certain nombre de moutons (passé en deuxième paramètre).
    """
    moutons = []
    for i in range(n):
        case_vide = False
        while not case_vide:
            ligne = randint(0, TAILLE - 1)
            colonne = randint(0, TAILLE - 1)
            case_vide = est_vide(monde, ligne, colonne)
        nouveau_mouton = creer_mouton(i, ligne, colonne)
        moutons.append(nouveau_mouton)
        monde[ligne][colonne] = nouveau_mouton
    return moutons

def creer_loups(grille, n):
    """
    Fonction permettant de peupler le monde (passé en premier paramètre)
    avec un certain nombre de loups (passé en deuxième paramètre).
    """
    loups = []
    for i in range(n):
        case_vide = False
        while not case_vide:
            ligne = randint(0, TAILLE - 1)
            colonne = randint(0, TAILLE - 1)
            case_vide = est_vide(grille, ligne, colonne)
        nouveau_loup = creer_loup(i, ligne, colonne)
        loups.append(nouveau_loup)
        grille[ligne][colonne] = nouveau_loup
    return loups

def creer_monde():
    """
    Fonction permettant de créer un monde sous la forme d'une grille
    carrée.
    """
    grille = []
    for num_ligne in range(TAILLE):
        grille.append([None] * TAILLE)
    return grille


def peupler_monde(grille, infos, nb_moutons, nb_loups):
    """
    Fonction permettant de peupler un monde (1er paramètre)
    avec un certain nombre moutons (2è paramètre) et un
    certain nombre de loups (3è paramètre).
    """
    liste_moutons = creer_moutons(grille, nb_moutons)
    liste_loups = creer_loups(grille, nb_loups)
    return liste_moutons, liste_loups

